/************************************************************************
* TaltOS base classes like time, output buffer, task control, task desc.*
* Author: Peter Eordegh                                                 *
************************************************************************/

#include "TaltOS_consts.hpp"

/************************************************************************
*                        class TaltOS_task_describer                    *
* function: this is task describer class used by TaltOS_task_control    *
* private class                                                         *
************************************************************************/

class TaltOS_task_describer {

	private :

	unsigned char occupied;
	void (*task_addr)(unsigned long);
	unsigned long ID;
	unsigned char priority;
	unsigned char state;
	unsigned long argument;

	public:

	void set_task(void(addr_in)(unsigned long),
				  unsigned long ID_in,
				  unsigned char priority_in,
				  unsigned char state_in,
				  unsigned long argument_in)
	{
		occupied=true;
		task_addr=addr_in;
		ID=ID_in;
		priority=priority_in;
		state=state_in;
		argument=argument_in;
	}

	void set_occupied(unsigned char set){occupied=set;}

	unsigned char is_occupied(){return occupied;}

	void set_state(unsigned char set){state=set;}

	unsigned char get_state(){return state;}

	void set_priority(unsigned char set){priority=set;}

	unsigned char get_priority(){return priority;}

	unsigned long get_ID(){return ID;}

	void run_me(){task_addr(argument);}

}; //end of TaltOS_task_describer

/************************************************************************
*                        class TaltOS_task_control                      *
* function: this controls the tasks: switch, create, delet tasks...     *
* public class                                                          *
************************************************************************/

class TaltOS_task_control {


	private:

	TaltOS_task_describer task_list[TALTOS_MAX_TASK_NUMBER];

	TaltOS_task_describer* actual_task;

	public:

	TaltOS_task_control()
	{
		long i;
		for(i=0;i<TALTOS_MAX_TASK_NUMBER;i++)
		{
			task_list[i].set_occupied(false);
		}
	}

	unsigned long actual_task_ID()
	{
		if (actual_task!=0)
		{return actual_task->get_ID();}
		else
		{return 0;}
	}

	unsigned char add_task(void(addr_in)(unsigned long),
						   unsigned long ID_in,
						   unsigned char priority_in,
						   unsigned char state_in,
						   unsigned long argument_in)
	{
		long i;
		for(i = 0;i < TALTOS_MAX_TASK_NUMBER;i++)
		{
			if (!task_list[i].is_occupied())
			{
				task_list[i].set_task(addr_in, ID_in, priority_in, state_in, argument_in);
				return true;
			}
		}
		return false;
	}

	unsigned char run_next_task()
	{
		int i,k = -1;
		for(i = 0;i < TALTOS_MAX_TASK_NUMBER;i++)
		{
			if (task_list[i].is_occupied())
			{
				if (k == -1) {k = i;}
				if (task_list[i].get_priority() > task_list[k].get_priority())
				{k = i;}
			}
		}
		if (k != -1)
		{
			actual_task = &task_list[k];
			actual_task -> run_me();
			actual_task -> set_occupied(false);
			actual_task = 0;
			return true;
		}
		else {return false;}

	}


}; //End of TaltOS_task_control

/************************************************************************
*                        class TaltOS_output_buffer                     *
* function: this is only an output buffer, can be used by serial bus    *
************************************************************************/

class TaltOS_output_buffer {


	private:

	unsigned char buffer[TALTOS_BUFFER_SIZE];
	unsigned long buffer_state;

	public:

	TaltOS_output_buffer()
	{
		for(long i=0;i<TALTOS_BUFFER_SIZE;i++)
		{
			buffer[i]=0;
			buffer_state=0;
		}
	}

	void clear_buffer()
	{
		for(long i=0;i<TALTOS_BUFFER_SIZE;i++)
		{
			buffer[i]=0;
			buffer_state=0;
		}
	}

	unsigned char put_char(unsigned char ch)
	{
		int i,k;

		for(i = 0; i < TALTOS_BUFFER_SIZE; i++)

		{
			k = (buffer_state + i);
			if (k >= 1024) {k -= 1024;}
			if (buffer[k] == 0)
			{
				buffer[k] = ch;
				return true;
			}
		}
		return false;
	}

	unsigned char put_string(const char* string)
	{
		for(unsigned short i = 0; string[i] != 0; i++)
		{
			if (!put_char(string[i])){return false;}
		}
		return true;
	}

	unsigned char get_char()
	{
		unsigned char ch = buffer[buffer_state];
		if (ch != 0)
		{
			buffer[buffer_state] = 0;
			buffer_state++;
			if (buffer_state >= 1024) {buffer_state=0;}
		}
		return ch;
	}
}; //End of TaltOS_output_buffer

/************************************************************************
*                       class TaltOS_time                               *
* function: it is the main class of time handeling in TaltOS            *
************************************************************************/


class TaltOS_time {


	private:

	unsigned long counter;

	unsigned long statistic[TALTOS_MAX_TASK_NUMBER];

	TaltOS_task_control* task_control;

	TaltOS_output_buffer* buffer;

	public:

	TaltOS_time()
	{
		counter = 0;
		for(long i = 0; i < TALTOS_MAX_TASK_NUMBER; i++)
		{
			statistic[i] = 0;
		}
	}

	void add_task_control_and_output_buffer(TaltOS_task_control* task_control_in,
											TaltOS_output_buffer* output_buffer_in)
	{
		task_control = task_control_in;
		buffer = output_buffer_in;
	}

	void tick()
	{
		counter++;
		statistic[task_control->actual_task_ID()]++;
		if (counter == 5000)
		{
			counter = 0;
			for(long i = 0;i < TALTOS_MAX_TASK_NUMBER;i++)
			{
				statistic[i] = 0;
			}
		}
	}

	void write_statistic()
	{
		long i,sum = 0;
		buffer->clear_buffer();
		for(i = 0; i < TALTOS_MAX_TASK_NUMBER; i++){sum += statistic[i];}
		buffer->put_char('*');
		buffer->put_char((sum / 1000)%10+'0');
		buffer->put_char((sum / 100)%10+'0');
		buffer->put_char((sum / 10)%10+'0');
		buffer->put_char((sum % 10)+'0');
		for(i = 0;i < TALTOS_MAX_TASK_NUMBER;i++)
		{
			if (statistic[i])
			{
				buffer->put_char(':');
				buffer->put_char((statistic[i] / 1000)%10+'0');
				buffer->put_char((statistic[i] / 100)%10+'0');
				buffer->put_char((statistic[i] / 10)%10+'0');
				buffer->put_char((statistic[i] % 10)+'0');
			}
		}
		buffer->put_char('*');
	}



}; //End of TaltOS_time


class TaltOS {

	private:

	TaltOS_output_buffer priv_output_buffer;
	TaltOS_task_control priv_task_control;
	TaltOS_time priv_time;

	public:

	TaltOS_output_buffer* output_buffer;
	TaltOS_task_control* task_control;
	TaltOS_time* time;

	TaltOS()
	{
		output_buffer = &priv_output_buffer;
		task_control = &priv_task_control;
		time = &priv_time;
		priv_time.add_task_control_and_output_buffer (&priv_task_control, &priv_output_buffer);
	}

	void start()
	{
		while ( 1 ) {priv_task_control.run_next_task();}
	}

}; //End of TaltOS
